//AmpelClientProtokoll.java                         MM 2015
import java.awt.*;                                 // Color
import java.io.*;                         // Reader, Writer
import java.net.*;                                // Socket

/**
 * Protokoll eines Clients, zeigt Nachrichten vom Server an
 * und schickt Nachrichten zum Server,
 * simuliert das Model in einer Kopie.
 */
class AmpelClientProtokoll
  extends AmpelModel
  implements Runnable
{
/**
 * Socket fuer Serververbindung
 */
  private Socket c;

/**
 * Eingabestrom vom Server
 */
  private ObjectInputStream vomServer;

/**
 * Ausgabestrom zum Server
 */
  private PrintWriter zumServer;

/**
 * Thread, wartet auf Serverinformationen.
 */
  private Thread t = null;

/**
 * Konstruktor, baut Datenstroeme zum/vom Server auf.
 * @param c Socket fuer Serververbindung
 */
  public AmpelClientProtokoll( Socket c)
  {
    try
    {
// Socket, Datenstroeme
      this.c = c;

      vomServer
      = new ObjectInputStream( c.getInputStream());

      zumServer
      = new PrintWriter( c.getOutputStream(), true);

// Liest Anfangszustand vom Server
      setLight( receive());

// Thread
      t = new Thread( this);
      t.start();

// Views GUI
      AmpelView2 view2 = new AmpelView2( this, "Ampel");
    }
    catch( Exception e)
    {
      System.out.println( "Fehler: " + e);
    }
  }

/**
 * Liest Ampelzustand vom Server
 */
  private boolean[] receive()
  throws Exception
  {
    boolean ryg[] = new boolean[3];
    for( int i = 0; i < 3; i++)
    {
      Boolean b = (Boolean)vomServer.readObject();
      ryg[i] = b.booleanValue();
    }
    return ryg;
  }

/**
 * Interaktion,
 * liest und verarbeitet Ampelzustand vom Server.
 */
  public void run()
  {
    try
    {
      while( true)
      {
        setLight( receive());
      }
    }
    catch( Exception e)
    {
      if( !t.isInterrupted())
      {
        System.out.println
        ( "\nVerbindung zum Server unterbrochen\n" + e);

        release();
      }
    }
  }

/**
 * Beendet AmpelClient, ueberschreibt Methode aus Model.
 * Meldet Abbruch dem Server,
 * bricht Verbindung zum Server ab, bricht Programm ab.
 */
  protected void release()
  {
// Thread beenden
    t.interrupt();

// Server informieren
    System.out.println( "Dialog beendet");
    zumServer.println( "quit");

// Clientsocket schliessen
    try
    {
      c.close();
      System.out.println( "Client abgemeldet");
    }
    catch( IOException e)
    {
      System.out.println
      ( "\nFehler beim Schliessen des Client\n" + e);
    }

// Programm beenden
    super.release();
  }

/* ------------------------------------------------------*/
                                        // service-Methoden
/**
 * Ueberschreibt das Schalten der Ampel,
 * schickt Nachricht an Server.
 */
  public void setLight()
  {
    zumServer.println( "set");    // Ampel wurde geschaltet
  }
}