//AmpelServerProtokoll.java                         MM 2015
import java.util.*;                             // Observer
import java.awt.*;                                 // Color
import java.io.*;                         // Reader, Writer
import java.net.*;                                // Socket

/**
 * Steuert Interaktion zwischen Server und Client,
 * ueberwacht AmpelModel (Original).
 */
class AmpelServerProtokoll
  extends Thread
  implements Observer
{
/**
 * Model, Original serverseitig.
 */
  private AmpelModel model;

/**
 * Socket fuer Clientverbindung.
 */
  private Socket s;

/**
 * Eingabestrom vom Client.
 */
  private BufferedReader vomClient;

/**
 * Ausgabestrom zum Client.
 */
  private ObjectOutputStream zumClient;

/**
 * Konstruktor,
 * baut Ueberwachungsmechanismus
 * und Datenstroeme zum/vom Client auf.
 * @param s Socket fuer Clientverbindung
 * @param m Modell auf dem Server
 */
  public AmpelServerProtokoll( Socket s, AmpelModel m)
  {
// Socket
    this.s = s;

// Modell, Ueberwachungsmechanismus
    model = m;
    model.addObserver( this);

    try
    {
// Datenstroeme
      vomClient = new BufferedReader
        ( new InputStreamReader( s.getInputStream()));

      zumClient = new ObjectOutputStream
        ( s.getOutputStream());

// Sendet Anfangszustand zum Client
      send( model.getLight());

// Thread, startet Dialog
      start();
    }
    catch( IOException e)
    {
      System.out.println( "IO Fehler: " + e);
    }
  }

/**
 * Sendet Ampelzustand zum Client.
 * @param ryg Ampelzustand (rot, gelb, gruen)
 */
  private void send( boolean[] ryg)
  {
    try
    {
      System.out.println( "Sende Ampeldaten");
      for( int i = 0; i < 3; i++)
        zumClient.writeObject( new Boolean( ryg[ i]));
    }
    catch(Exception e)
    {
      System.out.println
      ( "\nFehler beim Datentransfer\n" + e);
    }
  }

/**
 * Interaktion,
 * liest und verarbeitet Nachrichten vom Client,
 * "set"  .. schaltet die Ampel,
 * "quit" .. beendet das Programm.
 */
  public void run()
  {
    System.out.println( "Protokoll gestartet");

    try
    {
      String nachricht;
      do
      {
        nachricht = vomClient.readLine();

        if( nachricht.equalsIgnoreCase( "quit"))
        {
          release();                    // Transfer beenden
          return;
        }
 
        if( nachricht.equalsIgnoreCase( "set"))
        {
          System.out.println( "Ampel geschaltet");
          model.setLight();
        }
      } while( true);
    }
    catch( IOException e)
    {
      System.out.println
      ( "\nVerbindung zum Client unterbrochen\n" + e);
      release();
    }
  }

/**
 * Schliesst Verbindung zum Client,
 * setzt Model zurueck.
 */
  private void release()
  {
    System.out.println( "Protokoll beendet");

    try
    {
      s.close();
      model.deleteObserver( this);
      model = null;
    }
    catch( IOException e)
    {
      System.out.println
      ( "\nFehler beim Schliessen\n" + e);
    }
  }

/**
 * Ueberschreibt Interfacemethode update,
 * teilt Client Aenderung im Model mit.
 * @param m Model, welches Aenderungen meldet
 * @param o geaendertes Objekt
 */
  public void update( Observable m, Object o)
  {
    if( model == m)
    {
      System.out.println( "Datentransfer");
      send( model.getLight());
    }
  }
}